'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var core = require('@react-pdf-viewer/core');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

var NextIcon = function () {
    return (React__default['default'].createElement(core.Icon, { size: 16 },
        React__default['default'].createElement("path", { d: "M0.541,5.627L11.666,18.2c0.183,0.207,0.499,0.226,0.706,0.043c0.015-0.014,0.03-0.028,0.043-0.043\n                L23.541,5.627" })));
};

var PreviousIcon = function () {
    return (React__default['default'].createElement(core.Icon, { size: 16 },
        React__default['default'].createElement("path", { d: "M23.535,18.373L12.409,5.8c-0.183-0.207-0.499-0.226-0.706-0.043C11.688,5.77,11.674,5.785,11.66,5.8\n                L0.535,18.373" })));
};

var SearchIcon = function () {
    return (React__default['default'].createElement(core.Icon, { size: 16 },
        React__default['default'].createElement("path", { d: "M10.5,0.5c5.523,0,10,4.477,10,10s-4.477,10-10,10s-10-4.477-10-10S4.977,0.5,10.5,0.5z\n                M23.5,23.5\n                l-5.929-5.929" })));
};

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var EMPTY_KEYWORD_REGEXP = new RegExp(' ');

var PORTAL_OFFSET = { left: 0, top: 8 };
var SearchPopover = function (_a) {
    var doc = _a.doc, store = _a.store, onToggle = _a.onToggle;
    var l10n = React.useContext(core.LocalizationContext);
    var _b = React.useState(''), keyword = _b[0], setKeyword = _b[1];
    var _c = React.useState([]), found = _c[0], setFound = _c[1];
    var _d = React.useState(0), currentMatch = _d[0], setCurrentMatch = _d[1];
    var _e = React.useState(false), matchCase = _e[0], setMatchCase = _e[1];
    var textContents = React.useRef([]);
    var _f = React.useState(false), wholeWords = _f[0], setWholeWords = _f[1];
    var indexArr = Array(doc.numPages).fill(0).map(function (_, i) { return i; });
    var changeKeyword = function (e) {
        setKeyword(e.target.value);
    };
    var getTextContents = function () {
        var promises = indexArr.map(function (pageIndex) {
            return doc.getPage(pageIndex + 1).then(function (page) {
                return page.getTextContent();
            }).then(function (content) {
                var pageContent = content.items.map(function (item) { return item.str || ''; }).join('');
                return Promise.resolve({
                    pageContent: pageContent,
                    pageIndex: pageIndex,
                });
            });
        });
        return Promise.all(promises).then(function (data) {
            data.sort(function (a, b) { return a.pageIndex - b.pageIndex; });
            return Promise.resolve(data.map(function (item) { return item.pageContent; }));
        });
    };
    var keydownSearch = function (e) {
        if (e.keyCode !== 13 || !keyword) {
            return;
        }
        search(keyword, matchCase, wholeWords);
    };
    var changeMatchCase = function (e) {
        var isChecked = e.target.checked;
        setMatchCase(isChecked);
        if (keyword) {
            search(keyword, isChecked, wholeWords);
        }
    };
    var changeWholeWords = function (e) {
        var isChecked = e.target.checked;
        setWholeWords(isChecked);
        if (keyword) {
            search(keyword, matchCase, isChecked);
        }
    };
    var clearKeyword = function () {
        store.update('keyword', EMPTY_KEYWORD_REGEXP);
        setKeyword('');
        setCurrentMatch(0);
        setFound([]);
        setMatchCase(false);
        setWholeWords(false);
    };
    var close = function () {
        onToggle();
        clearKeyword();
    };
    var buildKeywordRegex = function (keywordParam, matchCaseParam, wholeWordsParam) {
        var source = wholeWordsParam ? " " + keywordParam + " " : keywordParam;
        var flags = matchCaseParam ? 'g' : 'gi';
        return new RegExp(source, flags);
    };
    var search = function (keywordParam, matchCaseParam, wholeWordsParam) {
        var regexp = buildKeywordRegex(keywordParam, matchCaseParam, wholeWordsParam);
        store.update('keyword', regexp);
        setCurrentMatch(0);
        setFound([]);
        var promise = (textContents.current.length === 0)
            ? getTextContents().then(function (response) {
                textContents.current = response;
                return Promise.resolve(response);
            })
            : Promise.resolve(textContents.current);
        promise.then(function (response) {
            var arr = [];
            response.forEach(function (item, pageIndex) {
                var numMatches = (item.match(regexp) || []).length;
                for (var matchIndex = 0; matchIndex < numMatches; matchIndex++) {
                    arr.push({
                        matchIndex: matchIndex,
                        pageIndex: pageIndex,
                    });
                }
            });
            setFound(arr);
            if (arr.length > 0) {
                setCurrentMatch(1);
                jumpToMatch(arr[0]);
            }
        });
    };
    var jumpToPreviousMatch = function () {
        var prev = currentMatch - 1;
        var updated = prev > 0 ? prev : found.length;
        setCurrentMatch(updated);
        jumpToMatch(found[updated - 1]);
    };
    var jumpToNextMatch = function () {
        var next = currentMatch + 1;
        var updated = next <= found.length ? next : 1;
        setCurrentMatch(updated);
        jumpToMatch(found[updated - 1]);
    };
    var jumpToMatch = function (match) {
        var jumpToPage = store.get('jumpToPage');
        if (jumpToPage) {
            jumpToPage(match.pageIndex);
        }
        store.update('match', match);
    };
    return (React__default['default'].createElement("div", { className: 'rpv-search-popover' },
        React__default['default'].createElement("div", { className: 'rpv-search-popover-input-counter' },
            React__default['default'].createElement("input", { className: 'rpv-search-popover-input', placeholder: (l10n && l10n.search ? l10n.search.enterToSearch : 'Enter to search'), type: "text", value: keyword, onChange: changeKeyword, onKeyDown: keydownSearch }),
            React__default['default'].createElement("div", { className: 'rpv-search-popover-counter' },
                currentMatch,
                "/",
                found.length)),
        React__default['default'].createElement("label", { className: 'rpv-search-popover-label' },
            React__default['default'].createElement("input", { className: 'rpv-search-popover-label-checkbox', checked: matchCase, type: "checkbox", onChange: changeMatchCase }),
            " ",
            l10n && l10n.search ? l10n.search.matchCase : 'Match case'),
        React__default['default'].createElement("label", { className: 'rpv-search-popover-label' },
            React__default['default'].createElement("input", { className: 'rpv-search-popover-label-checkbox', checked: wholeWords, type: "checkbox", onChange: changeWholeWords }),
            " ",
            l10n && l10n.search ? l10n.search.wholeWords : 'Whole words'),
        React__default['default'].createElement("div", { className: 'rpv-search-popover-footer' },
            React__default['default'].createElement("div", { className: 'rpv-search-popover-footer-item' },
                React__default['default'].createElement(core.Tooltip, { position: core.Position.BottomCenter, target: React__default['default'].createElement(core.Button, { onClick: jumpToPreviousMatch },
                        React__default['default'].createElement(PreviousIcon, null)), content: function () { return (l10n && l10n.search ? l10n.search.previousMatch : 'Previous match'); }, offset: PORTAL_OFFSET })),
            React__default['default'].createElement("div", { className: 'rpv-search-popover-footer-item' },
                React__default['default'].createElement(core.Tooltip, { position: core.Position.BottomCenter, target: React__default['default'].createElement(core.Button, { onClick: jumpToNextMatch },
                        React__default['default'].createElement(NextIcon, null)), content: function () { return (l10n && l10n.search ? l10n.search.nextMatch : 'Next match'); }, offset: PORTAL_OFFSET })),
            React__default['default'].createElement("div", { className: 'rpv-search-popover-footer-button' },
                React__default['default'].createElement(core.PrimaryButton, { onClick: close }, l10n && l10n.search ? l10n.search.close : 'Close')))));
};

var ShowSearchPopoverDecorator = function (_a) {
    var children = _a.children, onClick = _a.onClick;
    var l10n = React.useContext(core.LocalizationContext);
    var label = (l10n && l10n.search ? l10n.search.search : 'Search');
    var icon = React__default['default'].createElement(SearchIcon, null);
    return children({ icon: icon, label: label, onClick: onClick });
};

var TOOLTIP_OFFSET = { left: 0, top: 8 };
var ShowSearchPopoverButton = function (_a) {
    var onClick = _a.onClick;
    return (React__default['default'].createElement(ShowSearchPopoverDecorator, { onClick: onClick }, function (p) { return (React__default['default'].createElement(core.Tooltip, { position: core.Position.BottomCenter, target: React__default['default'].createElement(core.Button, { onClick: onClick }, p.icon), content: function () { return p.label; }, offset: TOOLTIP_OFFSET })); }));
};

var useDocument = function (store) {
    var _a = React.useState(store.get('doc')), currentDoc = _a[0], setCurrentDoc = _a[1];
    var handleDocumentChanged = function (doc) {
        setCurrentDoc(doc);
    };
    React.useEffect(function () {
        store.subscribe('doc', handleDocumentChanged);
        return function () {
            store.unsubscribe('doc', handleDocumentChanged);
        };
    }, []);
    return { currentDoc: currentDoc };
};

var PORTAL_OFFSET$1 = { left: 0, top: 8 };
var ShowSearchPopover = function (_a) {
    var children = _a.children, store = _a.store;
    var currentDoc = useDocument(store).currentDoc;
    var defaultChildren = function (props) { return (React__default['default'].createElement(ShowSearchPopoverButton, __assign({}, props))); };
    var render = children || defaultChildren;
    return (currentDoc
        ? (React__default['default'].createElement(core.Popover, { position: core.Position.BottomLeft, target: function (toggle) { return render({
                onClick: toggle,
            }); }, content: function (toggle) { return React__default['default'].createElement(SearchPopover, { doc: currentDoc, store: store, onToggle: toggle }); }, offset: PORTAL_OFFSET$1, closeOnClickOutside: false, closeOnEscape: true }))
        : React__default['default'].createElement(React__default['default'].Fragment, null));
};

var calculateOffset = function (children, parent) {
    var top = children.offsetTop;
    var left = children.offsetLeft;
    var p = children.parentElement;
    while (p && p !== parent) {
        top += p.offsetTop;
        left += p.offsetLeft;
        p = p.parentElement;
    }
    return {
        left: left,
        top: top,
    };
};

var removeNode = function (ele) {
    var parent = ele.parentNode;
    if (parent) {
        parent.removeChild(ele);
    }
};
var replaceNode = function (replacementNode, node) {
    removeNode(replacementNode);
    var parent = node.parentNode;
    if (parent) {
        parent.insertBefore(replacementNode, node);
    }
    removeNode(node);
};
var unwrap = function (ele) {
    var parent = ele.parentNode;
    if (!parent) {
        return;
    }
    var range = document.createRange();
    range.selectNodeContents(ele);
    replaceNode(range.extractContents(), ele);
    parent.normalize();
};

var wrap = function (ele, startOffset, endOffset) {
    var range = new Range();
    range.setStart(ele, startOffset);
    range.setEnd(ele, endOffset);
    var wrapper = document.createElement('span');
    range.surroundContents(wrapper);
    return wrapper;
};

var Tracker = function (_a) {
    var pageIndex = _a.pageIndex, store = _a.store;
    var _b = React.useState({
        matchIndex: -1,
        pageIndex: -1,
    }), match = _b[0], setMatch = _b[1];
    var _c = React.useState(EMPTY_KEYWORD_REGEXP), keywordRegexp = _c[0], setKeywordRegexp = _c[1];
    var _d = React.useState({
        pageIndex: pageIndex,
        scale: 1,
        status: core.TextLayerRenderStatus.PreRender,
    }), renderStatus = _d[0], setRenderStatus = _d[1];
    var unhighlightAll = function (containerEle) {
        var highlightNodes = containerEle.querySelectorAll('span.rpv-search-text-highlight');
        var total = highlightNodes.length;
        for (var i = 0; i < total; i++) {
            unwrap(highlightNodes[i]);
        }
    };
    var highlight = function (span) {
        var text = span.textContent;
        if (!keywordRegexp.source.trim() || !text) {
            return;
        }
        var startOffset = text.search(keywordRegexp);
        var firstChild = span.firstChild;
        if (startOffset === -1 || !firstChild) {
            return;
        }
        var endOffset = startOffset + keywordRegexp.source.length;
        var wrapper = wrap(firstChild, startOffset, endOffset);
        wrapper.classList.add('rpv-search-text-highlight');
    };
    var handleKeywordChanged = function (keyword) {
        if (keyword) {
            setKeywordRegexp(keyword);
        }
    };
    var handleMatchChanged = function (currentMatch) {
        setMatch(currentMatch);
    };
    var handleRenderStatusChanged = function (status) {
        if (!status.has(pageIndex)) {
            return;
        }
        var currentStatus = status.get(pageIndex);
        if (currentStatus) {
            setRenderStatus({
                ele: currentStatus.ele,
                pageIndex: pageIndex,
                scale: currentStatus.scale,
                status: currentStatus.status,
            });
        }
    };
    React.useEffect(function () {
        if (!keywordRegexp.source.trim() || !renderStatus.ele || renderStatus.status !== core.TextLayerRenderStatus.DidRender) {
            return;
        }
        var containerEle = renderStatus.ele;
        var spans = containerEle.childNodes;
        var numSpans = spans.length;
        unhighlightAll(containerEle);
        for (var i = 0; i < numSpans; i++) {
            var span = spans[i];
            highlight(span);
        }
        scrollToMatch();
    }, [keywordRegexp, match, renderStatus]);
    React.useEffect(function () {
        if (keywordRegexp.source.trim() === '' && renderStatus.ele && renderStatus.status === core.TextLayerRenderStatus.DidRender) {
            unhighlightAll(renderStatus.ele);
        }
    }, [keywordRegexp, renderStatus]);
    var scrollToMatch = function () {
        if (match.pageIndex !== pageIndex || !renderStatus.ele || renderStatus.status !== core.TextLayerRenderStatus.DidRender) {
            return;
        }
        var container = renderStatus.ele;
        var spans = container.querySelectorAll('.rpv-search-text-highlight');
        if (match.matchIndex < spans.length) {
            var span = spans[match.matchIndex];
            var top_1 = calculateOffset(span, container).top;
            var jump = store.get('jumpToDestination');
            if (jump) {
                jump(pageIndex, (container.getBoundingClientRect().height - top_1) / renderStatus.scale, renderStatus.scale);
            }
        }
    };
    React.useEffect(function () {
        store.subscribe('keyword', handleKeywordChanged);
        store.subscribe('match', handleMatchChanged);
        store.subscribe('renderStatus', handleRenderStatusChanged);
        return function () {
            store.unsubscribe('keyword', handleKeywordChanged);
            store.unsubscribe('match', handleMatchChanged);
            store.unsubscribe('renderStatus', handleRenderStatusChanged);
        };
    }, []);
    return React__default['default'].createElement(React__default['default'].Fragment, null);
};

var searchPlugin = function (props) {
    var store = core.createStore({
        renderStatus: new Map(),
    });
    var ShowSearchPopoverDecorator = function (props) { return (React__default['default'].createElement(ShowSearchPopover, __assign({}, props, { store: store }))); };
    var ShowSearchPopoverButtonDecorator = function () { return (React__default['default'].createElement(ShowSearchPopoverDecorator, null, function (props) { return React__default['default'].createElement(ShowSearchPopoverButton, __assign({}, props)); })); };
    var renderViewer = function (props) {
        var currentSlot = props.slot;
        if (currentSlot.subSlot) {
            currentSlot.subSlot.children = (React__default['default'].createElement(React__default['default'].Fragment, null,
                Array(props.doc.numPages).fill(0).map(function (_, index) { return (React__default['default'].createElement(Tracker, { key: index, pageIndex: index, store: store })); }),
                currentSlot.subSlot.children));
        }
        return currentSlot;
    };
    return {
        install: function (pluginFunctions) {
            store.update('jumpToDestination', pluginFunctions.jumpToDestination);
            store.update('jumpToPage', pluginFunctions.jumpToPage);
            store.update('keyword', props
                ? ((typeof props.keyword === 'string')
                    ? (props.keyword === '' ? EMPTY_KEYWORD_REGEXP : new RegExp(props.keyword))
                    : (props.keyword || EMPTY_KEYWORD_REGEXP))
                : EMPTY_KEYWORD_REGEXP);
        },
        renderViewer: renderViewer,
        uninstall: function (props) {
            var renderStatus = store.get('renderStatus');
            if (renderStatus) {
                renderStatus.clear();
            }
        },
        onDocumentLoad: function (props) {
            store.update('doc', props.doc);
        },
        onTextLayerRender: function (props) {
            var renderStatus = store.get('renderStatus');
            if (renderStatus) {
                renderStatus = renderStatus.set(props.pageIndex, props);
                store.update('renderStatus', renderStatus);
            }
        },
        ShowSearchPopover: ShowSearchPopoverDecorator,
        ShowSearchPopoverButton: ShowSearchPopoverButtonDecorator,
    };
};

exports.NextIcon = NextIcon;
exports.PreviousIcon = PreviousIcon;
exports.SearchIcon = SearchIcon;
exports.searchPlugin = searchPlugin;
